"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getHighlightedElementFromEventPath = exports.getDataAttributesFromEventPath = exports.dataToDatasetAttributeName = exports.DataAttribute = void 0;
exports.DataAttribute = {
    ElementCodename: 'data-kontent-element-codename',
    ComponentId: 'data-kontent-component-id',
    ItemId: 'data-kontent-item-id',
    LanguageCodename: 'data-kontent-language-codename',
    ProjectId: 'data-kontent-project-id',
};
var DataAttributeHierarchy = [
    exports.DataAttribute.ProjectId,
    exports.DataAttribute.LanguageCodename,
    exports.DataAttribute.ItemId,
    exports.DataAttribute.ComponentId,
    exports.DataAttribute.ElementCodename,
];
var DataAttributeParsingOrder = DataAttributeHierarchy.slice().reverse();
var OptionalDataAttributes = [exports.DataAttribute.ComponentId];
function getHigherDataAttributes(attributeName) {
    var attributeIndex = DataAttributeHierarchy.indexOf(attributeName);
    return DataAttributeHierarchy.slice(0, attributeIndex);
}
function dataToDatasetAttributeName(dataAttributeName) {
    return dataAttributeName.replace(/^data-/, '').replace(/-./g, function (x) { return x.toUpperCase()[1]; });
}
exports.dataToDatasetAttributeName = dataToDatasetAttributeName;
function getDataAttributesFromEventPath(event) {
    var e_1, _a, e_2, _b, e_3, _c;
    var path = event.composedPath();
    var parsed = new Map();
    try {
        for (var path_1 = __values(path), path_1_1 = path_1.next(); !path_1_1.done; path_1_1 = path_1.next()) {
            var node = path_1_1.value;
            if (!(node instanceof HTMLElement))
                continue;
            try {
                for (var DataAttributeParsingOrder_1 = (e_2 = void 0, __values(DataAttributeParsingOrder)), DataAttributeParsingOrder_1_1 = DataAttributeParsingOrder_1.next(); !DataAttributeParsingOrder_1_1.done; DataAttributeParsingOrder_1_1 = DataAttributeParsingOrder_1.next()) {
                    var attributeName = DataAttributeParsingOrder_1_1.value;
                    var datasetAttributeName = dataToDatasetAttributeName(attributeName);
                    var datasetAttributeValue = node.dataset[datasetAttributeName];
                    var isAttributeOptional = OptionalDataAttributes.includes(attributeName);
                    if (!datasetAttributeValue || parsed.has(attributeName))
                        continue;
                    // Higher attributes are attributes that are placed higher in the hierarchy, for example,
                    // for component id attribute those will be item id, language codename and project id.
                    var higherDataAttributes = getHigherDataAttributes(attributeName);
                    var areSomeHigherAttributesSet = higherDataAttributes.some(function (attr) { return parsed.has(attr); });
                    // If some higher attributes have already been parsed, it can mean two things:
                    // a) Current data attribute is optional and is not related to the current scope.
                    //    In that case it will be ignored.
                    //      Example:
                    //        <body data-kontent-project-id data-kontent-language-codename>
                    //          <div data-kontent-item-id data-kontent-component-id data-kontent-element-codename>
                    //            <!-- when parser comes to this element itemId and elementCodename are already parsed
                    //                 data-kontent-component-id is not related to them, so it should be ignored -->
                    //            <div data-kontent-item-id data-kontent-element-codename></div>
                    //          </div>
                    //        </body>
                    // b) Parsed data is related to some lower scope (or invalid) and should be cleared.
                    //      Example:
                    //        <body data-kontent-project-id data-kontent-language-codename>
                    //          <div data-kontent-item-id data-kontent-element-codename>
                    //            <div data-kontent-item-id>
                    //               <!-- if you click on this element, item id will be parsed, but it is not related to the
                    //                  data-kontent-element-codename above it, and this value must be cleared -->
                    //            </div>
                    //          </div>
                    //        </body>
                    if (areSomeHigherAttributesSet) {
                        if (isAttributeOptional)
                            continue;
                        try {
                            for (var higherDataAttributes_1 = (e_3 = void 0, __values(higherDataAttributes)), higherDataAttributes_1_1 = higherDataAttributes_1.next(); !higherDataAttributes_1_1.done; higherDataAttributes_1_1 = higherDataAttributes_1.next()) {
                                var attr = higherDataAttributes_1_1.value;
                                parsed.delete(attr);
                            }
                        }
                        catch (e_3_1) { e_3 = { error: e_3_1 }; }
                        finally {
                            try {
                                if (higherDataAttributes_1_1 && !higherDataAttributes_1_1.done && (_c = higherDataAttributes_1.return)) _c.call(higherDataAttributes_1);
                            }
                            finally { if (e_3) throw e_3.error; }
                        }
                    }
                    parsed.set(attributeName, datasetAttributeValue);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (DataAttributeParsingOrder_1_1 && !DataAttributeParsingOrder_1_1.done && (_b = DataAttributeParsingOrder_1.return)) _b.call(DataAttributeParsingOrder_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (path_1_1 && !path_1_1.done && (_a = path_1.return)) _a.call(path_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return parsed;
}
exports.getDataAttributesFromEventPath = getDataAttributesFromEventPath;
function getHighlightedElementFromEventPath(event) {
    var _a;
    var elementCodenameAttributeName = dataToDatasetAttributeName(exports.DataAttribute.ElementCodename);
    var highlightedElement = event.composedPath().find(function (element) {
        return element instanceof HTMLElement && element.dataset[elementCodenameAttributeName];
    });
    return (_a = highlightedElement) !== null && _a !== void 0 ? _a : null;
}
exports.getHighlightedElementFromEventPath = getHighlightedElementFromEventPath;
//# sourceMappingURL=dataAttributes.js.map