interface IParentMetadata {
    readonly isPositioned: boolean;
    readonly isContentClipped: boolean;
}
/**
 * Iterate through node ancestors and find an element which will be used as a parent for
 * the highlights container (highlights -> container -> parent). This element should either be
 * positioned (position is anything except static) or should have its content clipped. In case of
 * table element (td, th, table) it should be positioned or it will be ignored (even if its content is clipped).
 *
 * @param {HTMLElement} node
 * @returns {[HTMLElement, IParentMetadata] | [null, null]}
 */
export declare function getParentForHighlight(node: HTMLElement): [HTMLElement, IParentMetadata] | [null, null];
/**
 * Iterate through node ancestors until HTMLElement.offsetParent is reached and sum scroll offsets.
 *
 * @param {HTMLElement | null} node
 * @returns {[number, number]}
 *  where the first number is a totalScrollTop, and the second number is a totalScrollLeft.
 */
export declare function getTotalScrollOffset(node: HTMLElement | null): [number, number];
export {};
