var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { getParentForHighlight, getTotalScrollOffset } from '../utils/node';
export var HighlighterContainerTag = 'KONTENT-SMART-LINK-OVERLAY';
export var HighlighterElementTag = 'KONTENT-SMART-LINK-ELEMENT';
var HighlightRenderer = /** @class */ (function () {
    function HighlightRenderer() {
        var _this = this;
        this.destroy = function () {
            _this.clear();
            _this.defaultContainer.remove();
        };
        this.selectNode = function (node) {
            var highlight = _this.highlightByNode.get(node);
            if (highlight) {
                node.classList.add('kontent-smart-link__node--selected');
                highlight.classList.add('selected');
            }
        };
        this.deselectNode = function (node) {
            var highlight = _this.highlightByNode.get(node);
            if (highlight) {
                node.classList.remove('kontent-smart-link__node--selected');
                highlight.classList.remove('selected');
            }
        };
        this.render = function (nodes) {
            var e_1, _a, e_2, _b, e_3, _c;
            var _d;
            if (nodes.size === 0) {
                _this.clear();
            }
            else {
                var newHighlightByNode = new Map();
                try {
                    for (var nodes_1 = __values(nodes), nodes_1_1 = nodes_1.next(); !nodes_1_1.done; nodes_1_1 = nodes_1.next()) {
                        var node = nodes_1_1.value;
                        // Get size of the node and its position relative to viewport.
                        var nodeRect = node.getBoundingClientRect();
                        // This check is needed to prevent highlight rendering for the "flat" elements (height or/and width === 0),
                        // because those elements are basically invisible and cannot be clicked.
                        var isFlat = nodeRect.height === 0 || nodeRect.width === 0;
                        if (!isFlat) {
                            var _e = __read(getParentForHighlight(node), 2), parentElement = _e[0], parentMetadata = _e[1];
                            var highlight = (_d = _this.highlightByNode.get(node)) !== null && _d !== void 0 ? _d : _this.createHighlight(parentElement);
                            if (parentElement && parentMetadata) {
                                var parentRect = parentElement.getBoundingClientRect();
                                var container = _this.containerByParent.get(parentElement);
                                if (container && !parentMetadata.isPositioned) {
                                    // When parent element is not positioned it means that highlight
                                    // will be positioned relative to some other element. That is why we need
                                    // to keep in mind all of the scroll offsets on the way to this relative element.
                                    var _f = __read(getTotalScrollOffset(parentElement), 2), scrollOffsetTop_1 = _f[0], scrollOffsetLeft_1 = _f[1];
                                    container.style.height = parentElement.clientHeight + "px";
                                    container.style.width = parentElement.clientWidth + "px";
                                    container.style.top = parentElement.offsetTop - scrollOffsetTop_1 + "px";
                                    container.style.left = parentElement.offsetLeft - scrollOffsetLeft_1 + "px";
                                    if (parentMetadata.isContentClipped) {
                                        // When parent element is not positioned and its content is clipped
                                        // we need to hide overflow of the container as well to prevent
                                        // highlights from appearing for overflown content.
                                        container.classList.add('kontent-smart-link-overlay--restricted');
                                    }
                                }
                                // If the parent element is positioned and its content is clipped (hidden, scroll, auto, clipped),
                                // the parent element is an offset parent for the highlight and its scroll position can effect the position
                                // of the highlight, so we need to reckon with that.
                                var isPositionedAndClipped = parentMetadata.isPositioned && parentMetadata.isContentClipped;
                                var scrollOffsetTop = isPositionedAndClipped ? parentElement.scrollTop : 0;
                                var scrollOffsetLeft = isPositionedAndClipped ? parentElement.scrollLeft : 0;
                                highlight.style.top = nodeRect.top - parentRect.top + scrollOffsetTop + "px";
                                highlight.style.left = nodeRect.left - parentRect.left + scrollOffsetLeft + "px";
                            }
                            else {
                                // No parent element means that there is no positioned or clipped ancestor and that highlight will be
                                // placed into the body and page offset (page scroll) should be used.
                                highlight.style.top = nodeRect.top + window.pageYOffset + "px";
                                highlight.style.left = nodeRect.left + window.pageXOffset + "px";
                            }
                            highlight.style.width = nodeRect.width + "px";
                            highlight.style.height = nodeRect.height + "px";
                            // We are creating a new highlight by node map to be able to compare it with an old one to find out
                            // which nodes have been removed before renders and remove their highlights from the DOM.
                            newHighlightByNode.set(node, highlight);
                            _this.highlightByNode.delete(node);
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (nodes_1_1 && !nodes_1_1.done && (_a = nodes_1.return)) _a.call(nodes_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                try {
                    // All highlights that are left in the old highlightByNode map are the remnants of the old render
                    // and should be removed because their node has already been removed/or moved out of the viewport.
                    for (var _g = __values(_this.highlightByNode.entries()), _h = _g.next(); !_h.done; _h = _g.next()) {
                        var _j = __read(_h.value, 2), node = _j[0], highlight = _j[1];
                        highlight.remove();
                        _this.highlightByNode.delete(node);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_h && !_h.done && (_b = _g.return)) _b.call(_g);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
                try {
                    // All highlight containers that have no children can be removed because they are not used by any highlight.
                    for (var _k = __values(_this.containerByParent.entries()), _l = _k.next(); !_l.done; _l = _k.next()) {
                        var _m = __read(_l.value, 2), parent_1 = _m[0], container = _m[1];
                        if (container.children.length === 0) {
                            container.remove();
                            _this.containerByParent.delete(parent_1);
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (_l && !_l.done && (_c = _k.return)) _c.call(_k);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
                _this.highlightByNode = newHighlightByNode;
            }
        };
        this.clear = function () {
            var e_4, _a, e_5, _b;
            try {
                for (var _c = __values(_this.highlightByNode.entries()), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = __read(_d.value, 2), node = _e[0], highlight = _e[1];
                    highlight.remove();
                    _this.highlightByNode.delete(node);
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
                }
                finally { if (e_4) throw e_4.error; }
            }
            try {
                for (var _f = __values(_this.containerByParent.entries()), _g = _f.next(); !_g.done; _g = _f.next()) {
                    var _h = __read(_g.value, 2), parent_2 = _h[0], container = _h[1];
                    container.remove();
                    _this.containerByParent.delete(parent_2);
                }
            }
            catch (e_5_1) { e_5 = { error: e_5_1 }; }
            finally {
                try {
                    if (_g && !_g.done && (_b = _f.return)) _b.call(_f);
                }
                finally { if (e_5) throw e_5.error; }
            }
            _this.highlightByNode = new Map();
            _this.containerByParent = new Map();
            _this.defaultContainer.innerHTML = '';
        };
        this.createHighlight = function (parent) {
            var highlight = document.createElement(HighlighterElementTag);
            var container = _this.createContainerIfNotExist(parent);
            container.appendChild(highlight);
            return highlight;
        };
        this.createContainerIfNotExist = function (parent) {
            if (!parent) {
                return _this.defaultContainer;
            }
            if (_this.containerByParent.has(parent)) {
                return _this.containerByParent.get(parent);
            }
            var container = document.createElement(HighlighterContainerTag);
            parent.appendChild(container);
            _this.containerByParent.set(parent, container);
            return container;
        };
        this.containerByParent = new Map();
        this.highlightByNode = new Map();
        this.defaultContainer = HighlightRenderer.createDefaultContainer();
    }
    HighlightRenderer.createDefaultContainer = function () {
        var container = document.createElement(HighlighterContainerTag);
        container.classList.add('kontent-smart-link-default-overlay');
        window.document.body.appendChild(container);
        return container;
    };
    return HighlightRenderer;
}());
export { HighlightRenderer };
//# sourceMappingURL=HighlightRenderer.js.map